@@ -81,37 +81,57 @@ import_delim <- function(file, which = 1, sep = "auto", header = "auto", strings
     import_delim(file = file, sep = if (sep == "|") "auto" else sep, dec = dec, ...)
 }
 
-#' @export
-.import.rio_fwf <-
-    function(file,
-             which = 1,
-             widths,
-             header = FALSE,
-             col.names,
-             comment = "#",
-             readr = lifecycle::deprecated(),
-             progress = getOption("verbose", FALSE),
-             ...) {
-        if (lifecycle::is_present(readr)) {
-            lifecycle::deprecate_warn(when = "0.5.31", what = "import(readr)", details = "fwt will always be read without `readr`. The parameter `readr` will be dropped in v2.0.0.")
-        }
-        if (missing(widths)) {
-            stop("Import of fixed-width format data requires a 'widths' argument. See ? read.fwf().")
-        }
-        if (!missing(col.names)) {
-            read.fwf2(file = file, widths = widths, header = header, col.names = col.names, ...)
-        } else {
-            read.fwf2(file = file, widths = widths, header = header, ...)
-        }
+.get_col_position <- function(file, widths, col_names, col_position) {
+    if (missing(col_names)) {
+        col_names <- NULL
     }
+    if (!is.null(col_position)) {
+        return(col_position)
+    }
+    if (is.list(widths) && isFALSE(c("begin", "end") %in% names(widths))) {
+        return(readr::fwf_widths(widths, col_names = col_names))
+    }
+    if (isTRUE(is.numeric(widths))) {
+        return(readr::fwf_widths(abs(widths), col_names = col_names))
+    }
+    return(readr::fwf_empty(file = file, col_names = col_names))
+}
+
+.fix_col_types <- function(col_types, widths) {
+    if (isFALSE(is.numeric(widths))) {
+        return(col_types)
+    }
+    col_types <- rep("?", length(widths))
+    col_types[widths < 0] <- "?"
+    col_types <- paste0(col_types, collapse = "")
+    return(col_types)
+}
 
 #' @export
-.import.rio_r <- function(file, which = 1, ...) {
+.import.rio_fwf <- function(file, which = 1, widths = NULL, col_names, col_types = NULL, col_positions, col.names, col_position = NULL, comment = "#", ...) {
+    if (!is.null(widths)) {
+        lifecycle::deprecate_warn(when = "1.0.2", what = "import(widths)", details = "`widths` is kept for backward compatibility. Please use `col_positions` or unset `widths` to allow automatic guessing, see `?readr::read_fwf`. The parameter `widths` will be dropped in v2.0.0.")
+    }
+    if (!missing(col.names)) {
+        lifecycle::deprecate_warn(when = "1.0.2", what = "import(widths)", details = "`col.names` is kept for backward compatibility. Please use `col_names`. The parameter `col.names` will be dropped in v2.0.0.")
+        col_names <- col.names
+    }
+    col_positions <- .get_col_position(file = file, widths = widths, col_names = col_names, col_position = col_position)
+    if (!is.null(widths) && !is.null(col_types)) {
+        col_types <- .fix_col_types(col_types, widths)
+    }
+    .docall(readr::read_fwf, ..., args = list(file = file, col_positions = col_positions, col_types = col_types, comment = comment))
+}
+
+#' @export
+.import.rio_r <- function(file, which = 1, trust = getOption("rio.import.trust", default = NULL), ...) {
+    .check_trust(trust, format = ".R")
     .docall(dget, ..., args = list(file = file))
 }
 
 #' @export
-.import.rio_dump <- function(file, which = 1, envir = new.env(), ...) {
+.import.rio_dump <- function(file, which = 1, envir = new.env(), trust = getOption("rio.import.trust", default = NULL), ...) {
+    .check_trust(trust, format = "dump")
     source(file = file, local = envir)
     if (missing(which)) {
         if (length(ls(envir)) > 1) {
@@ -127,23 +147,29 @@ import_delim <- function(file, which = 1, sep = "auto", header = "auto", strings
 }
 
 #' @export
-.import.rio_rds <- function(file, which = 1, ...) {
+.import.rio_rds <- function(file, which = 1, trust = getOption("rio.import.trust", default = NULL), ...) {
+    .check_trust(trust, format = "RDS")
     readRDS(file = file)
 }
 
 #' @export
-.import.rio_rdata <- function(file, which = 1, envir = new.env(), ...) {
+.import.rio_rdata <- function(file, which = 1, envir = new.env(), trust = getOption("rio.import.trust", default = NULL), .return_everything = FALSE, ...) {
+    .check_trust(trust, format = "RData")
     load(file = file, envir = envir)
+    if (isTRUE(.return_everything)) {
+        ## for import_list()
+        return(as.list(envir))
+    }
     if (missing(which)) {
         if (length(ls(envir)) > 1) {
-            warning("Rdata file contains multiple objects. Returning first object.")
+          warning("Rdata file contains multiple objects. Returning first object.")
         }
-        which <- 1
+      which <- 1
     }
     if (is.numeric(which)) {
-        get(ls(envir)[which], envir)
+      get(ls(envir)[which], envir)
     } else {
-        get(ls(envir)[grep(which, ls(envir))[1]], envir)
+      get(ls(envir)[grep(which, ls(envir))[1]], envir)
     }
 }
 
@@ -152,6 +178,7 @@ import_delim <- function(file, which = 1, sep = "auto", header = "auto", strings
 
 #' @export
 .import.rio_feather <- function(file, which = 1, ...) {
+    .check_pkg_availability("arrow")
     .docall(arrow::read_feather, ..., args = list(file = file))
 }
 
@@ -386,6 +413,7 @@ extract_html_row <- function(x, empty_value) {
 
 #' @export
 .import.rio_parquet <- function(file, which = 1, ...) {
+    .check_pkg_availability("arrow")
     .docall(arrow::read_parquet, ..., args = list(file = file, as_data_frame = TRUE))
 }
 
